package net.wanji.opt.service.impl;


import com.alibaba.fastjson.JSONObject;
import lombok.extern.slf4j.Slf4j;
import net.wanji.common.enums.BaseEnum;
import net.wanji.common.enums.TurnConvertEnum;
import net.wanji.common.framework.dubbointerface.impl.BaseDubboInterfaceImpl;
import net.wanji.common.framework.exception.DubboProviderException;
import net.wanji.common.framework.mapper.BaseInterfaceMapper;
import net.wanji.common.gts.Tools;
import net.wanji.common.utils.tool.DateUtil;
import net.wanji.databus.po.CrossDataHistPO;
import net.wanji.databus.po.CrossDirDataHistPO;
import net.wanji.databus.po.CrossTurnDataHistPO;
import net.wanji.databus.po.CrossTurnInfoPO;
import net.wanji.opt.bo.BottomMenuBO;
import net.wanji.opt.common.EsDateIndexUtil;
import net.wanji.opt.common.enums.TimeGranularityEnum;
import net.wanji.opt.constant.ServiceLevelEnum;
import net.wanji.opt.dao.mapper.GreenwaveHistoryMapper;
import net.wanji.opt.entity.GreenwaveHist;
import net.wanji.opt.po.base.CrossLaneDataHistPoExtend;
import net.wanji.opt.service.GreenwaveHistProvider;
import net.wanji.opt.service.MainlineEvaluateService;
import net.wanji.opt.vo.GreenWaveRunStateVO;
import org.apache.commons.lang3.StringUtils;
import org.apache.dubbo.config.annotation.DubboService;
import org.joda.time.DateTime;
import org.joda.time.Seconds;
import org.joda.time.format.DateTimeFormat;
import org.joda.time.format.DateTimeFormatter;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Comparator;
import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.Random;
import java.util.Set;
import java.util.TreeMap;
import java.util.TreeSet;
import java.util.stream.Collectors;


/**
 * <p>
 * 绿波历史数据
 * </p>
 *
 * @Author fengyi
 * @Date 2024-11-18
 */
@Slf4j
@Component
@DubboService
public class GreenwaveHistProviderImpl extends BaseDubboInterfaceImpl<GreenwaveHist> implements GreenwaveHistProvider {

    @Resource
    private GreenwaveHistoryMapper greenwaveHistoryMapper;
    @Resource
    private MainlineEvaluateService mainlineEvaluateService;

    @Override
    public BaseInterfaceMapper<GreenwaveHist> getBaseInterfaceMapper() {
        return this.greenwaveHistoryMapper;
    }


    @Override
    public JSONObject findStatisticIndex(String greenId, String startTime, String endTime, String type) {
        Map<String, Object> params = new HashMap<>();
        params.put("greenId", greenId);
        params.put("startDate", startTime);
        params.put("endDate", endTime);

        List<GreenwaveHist> list = greenwaveHistoryMapper.findStatisticIndex(params);
        list = list.stream().sorted(Comparator.comparing(o -> o.getStartTime())).collect(Collectors.toList());

        //存放时段
        Set<String> sortedSet = new TreeSet<>();

        //存放所有数据
        List<Map<String, Object>> allList = new ArrayList<>();
        //按时间分组分组
        Map<String, List<GreenwaveHist>> groupByDir = list.stream().collect(Collectors.groupingBy(o -> o.getRoadDirection(), TreeMap::new, Collectors.toList()));

        List<Map<String, Object>> turnList = new ArrayList<>();

        for (Map.Entry<String, List<GreenwaveHist>> entry : groupByDir.entrySet()) {
            String key = entry.getKey();

            //按方向排序
            List<GreenwaveHist> value = entry.getValue().stream().sorted(Comparator.comparing(o -> o.getStartTime())).collect(Collectors.toList());

            Map<String, Object> mapList = new HashMap<>();
            mapList.put("direction", key);
            mapList.put("list", value);
            allList.add(mapList);

            for (GreenwaveHist po : value) {
                Map<String, Object> turnMap = new HashMap<>();
                turnMap.put("direction", po.getDirection());
                if (!turnList.contains(turnMap)) {
                    turnList.add(turnMap);
                }
                //提取时间
                sortedSet.add(DateUtil.formatDate(po.getStartTime(), "HH:mm"));
            }
        }

        JSONObject jsonObject = new JSONObject();
        jsonObject.put("type", type);
        jsonObject.put("timeList", sortedSet);
        jsonObject.put("dataList", allList);
        jsonObject.put("directionList", new int[]{0, 1, 2});

        return jsonObject;

    }

    @Override
    public List<Map<String, Object>> findGreenCross(Integer greenId) throws DubboProviderException {
        Map<String, Object> params = new HashMap<>();
        params.put("greenId", greenId);

        List<Map<String, Object>> list = greenwaveHistoryMapper.findGreenCross(params);
        String dirs = list.stream().map(o -> o.get("inDir").toString()).distinct().collect(Collectors.joining(","));


        return list;
    }

    @Override
    public JSONObject findUnCoordinateCrossIndex(String crossId, String greenDir, String startTime, String endTime) {
        Map<String, Object> params = new HashMap<>();
        params.put("crossId", crossId);
        params.put("startDate", startTime);
        params.put("endDate", endTime);
        params.put("greenDir", greenDir);

        List<GreenwaveHist> list = greenwaveHistoryMapper.findUnCoordinateCrossIndex(params);

        list = list.stream().sorted(Comparator.comparing(o -> o.getStartTime())).collect(Collectors.toList());

        //存放时段
        Set<String> sortedSet = new TreeSet<>();

        //存放所有数据
        List<Map<String, Object>> allList = new ArrayList<>();
        //按时间分组分组
        Map<Integer, List<GreenwaveHist>> groupByDir = list.stream().collect(Collectors.groupingBy(o -> o.getDirection(), TreeMap::new, Collectors.toList()));

        List<Map<String, Object>> turnList = new ArrayList<>();

        for (Map.Entry<Integer, List<GreenwaveHist>> entry : groupByDir.entrySet()) {
            Integer key = entry.getKey();

            //按方向排序
            List<GreenwaveHist> value = entry.getValue().stream().sorted(Comparator.comparing(o -> o.getStartTime())).collect(Collectors.toList());

            Map<String, Object> mapList = new HashMap<>();
            mapList.put("direction", key);
            mapList.put("list", value);
            allList.add(mapList);

            for (GreenwaveHist po : value) {
                Map<String, Object> turnMap = new HashMap<>();
                turnMap.put("direction", po.getDirection());
                if (!turnList.contains(turnMap)) {
                    turnList.add(turnMap);
                }
                //提取时间
                sortedSet.add(DateUtil.formatDate(po.getStartTime(), "HH:mm"));
            }
        }

        JSONObject jsonObject = new JSONObject();
        jsonObject.put("timeList", sortedSet);
        jsonObject.put("dataList", allList);

        return jsonObject;

    }

    @Override
    public JSONObject findGreenWaveCrossDirIndex(String crossId, String startTime, String endTime, String groupType) {
        Map<String, Object> params = new HashMap<>();
        params.put("crossId", crossId);
        params.put("startDate", startTime);
        params.put("endDate", endTime);
        params.put("groupType", groupType);

        List<CrossDirDataHistPO> list = greenwaveHistoryMapper.findGreenWaveCrossDirIndex(params);

        list = list.stream().sorted(Comparator.comparing(o -> o.getStartTime())).collect(Collectors.toList());

        //存放时段
        Set<String> sortedSet = new TreeSet<>();

        //存放所有数据
        List<Map<String, Object>> allList = new ArrayList<>();
        //按时间分组分组
        Map<Integer, List<CrossDirDataHistPO>> groupByDir = list.stream().collect(Collectors.groupingBy(o -> o.getDirType(), TreeMap::new, Collectors.toList()));

        List<Map<String, Object>> turnList = new ArrayList<>();

        for (Map.Entry<Integer, List<CrossDirDataHistPO>> entry : groupByDir.entrySet()) {
            Integer key = entry.getKey();

            //按方向排序
            List<CrossDirDataHistPO> value = entry.getValue().stream().sorted(Comparator.comparing(o -> o.getStartTime())).collect(Collectors.toList());

            Map<String, Object> mapList = new HashMap<>();
            mapList.put("direction", key);
            mapList.put("list", value);
            allList.add(mapList);

            for (CrossDirDataHistPO po : value) {
                Map<String, Object> turnMap = new HashMap<>();
                turnMap.put("direction", po.getDirType());
                if (!turnList.contains(turnMap)) {
                    turnList.add(turnMap);
                }
                //提取时间
                sortedSet.add(DateUtil.formatDate(po.getStartTime(), "HH:mm"));
            }
        }

        JSONObject jsonObject = new JSONObject();
        jsonObject.put("timeList", sortedSet);
        jsonObject.put("dataList", allList);
        jsonObject.put("dirList", groupByDir.keySet());

        return jsonObject;
    }


    @Override
    public JSONObject findGreenWaveCrossIndex(String crossId, String startTime, String endTime, String groupType) {
        Map<String, Object> params = new HashMap<>();
        params.put("crossId", crossId);
        params.put("startDate", startTime);
        params.put("endDate", endTime);
        params.put("groupType", groupType);

        List<CrossDataHistPO> list = greenwaveHistoryMapper.findGreenWaveCrossIndex(params);

        list = list.stream().sorted(Comparator.comparing(o -> o.getStartTime())).collect(Collectors.toList());

        //存放时段
        Set<String> sortedSet = new TreeSet<>();

        for (CrossDataHistPO po : list) {
            //提取时间
            sortedSet.add(DateUtil.formatDate(po.getStartTime(), "HH:mm"));
        }

        JSONObject jsonObject = new JSONObject();
        jsonObject.put("timeList", sortedSet);
        jsonObject.put("dataList", list);

        return jsonObject;
    }

    @Override
    public List<GreenWaveRunStateVO> findGreenWaveRunState(Integer greenId, String crossId, String startTime, String endTime, String eventType) {
        Map<String, Object> params = new HashMap<>();
        params.put("greenId", greenId);
        params.put("startDate", startTime);
        params.put("endDate", endTime);
        params.put("crossId", crossId);

        if (Objects.nonNull(eventType)) {
            String[] arr = StringUtils.split(eventType, ",");
            params.put("eventTypeList", Arrays.asList(arr));
        }

        int[] stateList = new int[]{705, 706};

        List<GreenWaveRunStateVO> list = greenwaveHistoryMapper.findGreenWaveRunState(params);
        Map<Integer, GreenWaveRunStateVO> groupList = list.stream().collect(Collectors.toMap(o -> o.getState(), o -> o));


        DateTime startDateTime = DateTime.parse(startTime, DateTimeFormat.forPattern("yyyy-MM-dd HH:mm:ss"));
        DateTime endDateTime = DateTime.parse(endTime, DateTimeFormat.forPattern("yyyy-MM-dd HH:mm:ss"));

        for (int state : stateList) {
            if (!groupList.containsKey(state)) {
                GreenWaveRunStateVO tmp = new GreenWaveRunStateVO();
                tmp.setState(state);
                tmp.setGreenId(greenId);
                list.add(tmp);
            }
        }
        int totalTime = Seconds.secondsBetween(startDateTime, endDateTime).getSeconds();
        for (GreenWaveRunStateVO vo : list) {
//            if (vo.getDuration()<=0) {
//                vo.setDuration(getRandomValue(totalTime / 10, 120));
//            }
            vo.setTotalTime(totalTime);
        }

        return list;
    }

    @Override
    public JSONObject findGreenWaveCrossTurnIndex(String crossId, String startTime, String endTime, String groupType) throws DubboProviderException {
        Map<String, Object> params = new HashMap<>();
        params.put("crossId", crossId);
        params.put("startDate", startTime);
        params.put("endDate", endTime);
        params.put("groupType", groupType);

        List<CrossTurnDataHistPO> list = greenwaveHistoryMapper.findGreenWaveCrossTurnIndex(params);

        list = list.stream().sorted(Comparator.comparing(o -> o.getStartTime())).collect(Collectors.toList());

        //存放时段
        Set<String> sortedSet = EsDateIndexUtil.getTimeGranularityAxis(groupType, startTime, endTime);
        //======================================================================================================//
        //存放所有数据
        List<Map<String, Object>> allList = new ArrayList<>();
        //按时间分组分组
        Map<String, List<CrossTurnDataHistPO>> groupByDir = list.stream().collect(Collectors.groupingBy(o -> o.getInDir() + "_" + o.getTurnType(), TreeMap::new, Collectors.toList()));


        for (Map.Entry<String, List<CrossTurnDataHistPO>> entry : groupByDir.entrySet()) {
            String key = entry.getKey();
            String[] sps = key.split("_");
            Integer dirType = Integer.parseInt(sps[0]);
            String dirName = BaseEnum.SignalDirectionEnum.getNameByCode(dirType);
            String turnDesc = TurnConvertEnum.getDescByKey(Integer.valueOf(sps[1]));
            String dirTurnDesc = dirName + "进口" + turnDesc;

            //按方向排序
            List<CrossTurnDataHistPO> value = entry.getValue().stream().sorted(Comparator.comparing(o -> o.getStartTime())).collect(Collectors.toList());
            //有实际数据的时刻点
            Set<String> timeList = value.stream().map(po -> DateUtil.formatDate(po.getStartTime(), EsDateIndexUtil.YMD_HM_FORMATTER)).collect(Collectors.toSet());
            //补充缺少时段数据，保留时段字段默认值
            for (String timeSec : sortedSet) {
                if (!timeList.contains(timeSec)) {
                    CrossTurnDataHistPO tmp = new CrossTurnDataHistPO();
                    tmp.setStartTime(DateTime.parse(timeSec, DateTimeFormat.forPattern(EsDateIndexUtil.YMD_HM_FORMATTER)).toDate());
                    value.add(tmp);
                }
            }
            value = value.stream().sorted(Comparator.comparing(CrossTurnDataHistPO::getStartTime)).collect(Collectors.toList());

//            sortedSet.clear();
//            value.forEach(o->sortedSet.add(DateUtil.formatDate(o.getStartTime(),"HH:mm")));

            Map<String, Object> mapList = new HashMap<>();
            mapList.put("dirTurnCode", key);
            mapList.put("dirTurnDesc", dirTurnDesc);
            mapList.put("list", value);
            allList.add(mapList);

        }

        JSONObject jsonObject = new JSONObject();
        jsonObject.put("timeList", sortedSet.stream().map(o -> o.substring(o.indexOf(" ") + 1, o.lastIndexOf(":"))));
        jsonObject.put("dataList", allList);
        jsonObject.put("dirList", groupByDir.keySet());


        return jsonObject;
    }

    @Override
    public JSONObject findGreenWaveCrossLaneIndex(String crossId, String startTime, String endTime, String groupType, Integer objectType) throws DubboProviderException {
        Map<String, Object> params = new HashMap<>();
        params.put("crossId", crossId);
        params.put("startDate", startTime);
        params.put("endDate", endTime);
        params.put("groupType", groupType);
        params.put("objectType", objectType);

        //存放时段
        Set<String> sortedSet = EsDateIndexUtil.getTimeGranularityAxis(groupType, startTime, endTime);
        //======================================================================================================//
        //存放所有数据
        List<Map<String, Object>> allList = new ArrayList<>();

        List<CrossLaneDataHistPoExtend> list = greenwaveHistoryMapper.findGreenWaveCrossLaneIndex(params);



        list = list.stream().sorted(Comparator.comparing(o -> o.getStartTime())).collect(Collectors.toList());
        //按时间分组分组
        Map<String, List<CrossLaneDataHistPoExtend>> groupByDir = list.stream().collect(Collectors.groupingBy(o -> o.getCrossId() + "_" + o.getLaneId() + "_" + o.getDirType() + "_" + o.getTurnType(), TreeMap::new, Collectors.toList()));

        Set<String> hasDataScopeList = new HashSet<>();

        for (Map.Entry<String, List<CrossLaneDataHistPoExtend>> entry : groupByDir.entrySet()) {
            String key = entry.getKey();
            String[] sps = key.split("_");
            String laneId = sps[1];
            String laneNo = laneId.substring(laneId.length() - 2);
            Integer dirType = null;
            String dirName = null;
            String turnDesc = null;
            String dirTurnDesc = null;

            Map<String, Object> mapList = new HashMap<>();

            if (Objects.equals(3, objectType)) {
                mapList.put("laneId", laneId);
                mapList.put("laneNo", laneNo);
                dirType = Integer.parseInt(sps[2]);
                dirName = BaseEnum.SignalDirectionEnum.getNameByCode(dirType);
                turnDesc = TurnConvertEnum.getDescByKey(Integer.valueOf(sps[3]));
                dirTurnDesc = dirName + "进口" + turnDesc + laneNo + "车道";
            } else if (Objects.equals(2, objectType)) {
                dirType = Integer.parseInt(sps[2]);
                dirName = BaseEnum.SignalDirectionEnum.getNameByCode(dirType);
                turnDesc = TurnConvertEnum.getDescByKey(Integer.valueOf(sps[3]));
                dirTurnDesc = dirName + "进口" + turnDesc;
            } else if (Objects.equals(1, objectType)) {
                dirType = Integer.parseInt(sps[2]);
                dirName = BaseEnum.SignalDirectionEnum.getNameByCode(dirType);
                dirTurnDesc = dirName + "进口";
            }

            hasDataScopeList.add(dirTurnDesc);

            //补充缺少时段数据，保留时段字段默认值
            List<CrossLaneDataHistPoExtend> value = this.processData(entry, groupType, sortedSet, startTime);



            mapList.put("scopeCode", key);
            mapList.put("scopeName", dirTurnDesc);
            mapList.put("list", value);
            allList.add(mapList);

        }
        BottomMenuBO bo = new BottomMenuBO();
        bo.setCrossId(crossId);
        bo.setScope(objectType);
        List<String> scopeList = mainlineEvaluateService.bottomMenu(bo);

        if (Objects.nonNull(scopeList)) {
            //补充一条时段都没有的
            for (String scopeName1 : scopeList) {
                if (!hasDataScopeList.contains(scopeName1)) {
                    Map<String, Object> mapList = new HashMap<>();
                    mapList.put("scopeCode", "");
                    mapList.put("scopeName", scopeName1);
                    List<CrossLaneDataHistPoExtend> tmpList = new ArrayList<>();
                    //补充所有时段字段对象
                    for (String time : sortedSet) {
                        CrossLaneDataHistPoExtend po = new CrossLaneDataHistPoExtend();
                        try {
                            if (Objects.equals("4", groupType)) {
                                //天粒度
                                String st = time + "000000";
                                po.setStartTime(DateUtil.parse(st, "yyyyMMddHHmmss"));

                            } else {
                                String st = startTime.substring(0, startTime.lastIndexOf(" ") + 1) + time + ":00";
                                po.setStartTime(DateUtil.parse(st, "yyyy-MM-dd HH:mm:ss"));
                            }
                        } catch (ParseException e) {
                            e.printStackTrace();
                        }
                        tmpList.add(po);
                        po.setTimeAxis(time);
                    }
                    mapList.put("list", tmpList);
                    allList.add(mapList);
                }
            }
        }

        JSONObject jsonObject = new JSONObject();
        jsonObject.put("timeList", sortedSet);
        jsonObject.put("dataList", allList);
        jsonObject.put("scopeList", scopeList);

        return jsonObject;
    }

    @Override
    public JSONObject findCrossExitRoadLaneInfo(String crossId, String startTime, String endTime, String groupType) {
        Map<String, Object> params = new HashMap<>();
        params.put("crossId", crossId);
        params.put("startDate", startTime);
        params.put("endDate", endTime);
        params.put("groupType", groupType);

        //存放时段
        Set<String> sortedSet = EsDateIndexUtil.getTimeGranularityAxis(groupType, startTime, endTime);
        //======================================================================================================//
        //存放所有数据
        List<Map<String, Object>> allList = new ArrayList<>();

        List<CrossLaneDataHistPoExtend> list = greenwaveHistoryMapper.findCrossExitRoadLaneIndex(params);

        list = list.stream().sorted(Comparator.comparing(o -> o.getStartTime())).collect(Collectors.toList());
        //按时间分组分组
        Map<String, List<CrossLaneDataHistPoExtend>> groupByDir = list.stream().collect(Collectors.groupingBy(o -> o.getCrossId() + "_" + o.getLaneId() + "_" + o.getDirType() + "_" + o.getTurnType(), TreeMap::new, Collectors.toList()));

        Set<String> scopeList = new TreeSet<>();

        for (Map.Entry<String, List<CrossLaneDataHistPoExtend>> entry : groupByDir.entrySet()) {
            String key = entry.getKey();
            String[] sps = key.split("_");

            String laneId = sps[1];
            String laneNo = laneId.substring(laneId.length() - 2);
            Integer dirType = Integer.parseInt(sps[2]);
            String dirName = null;
            String turnDesc = null;
            String dirTurnDesc = null;

            Map<String, Object> mapList = new HashMap<>();

            mapList.put("laneId", laneId);
            mapList.put("laneNo", laneNo);
            dirName = BaseEnum.SignalDirectionEnum.getNameByCode(dirType);
            turnDesc = TurnConvertEnum.getDescByKey(Integer.valueOf(sps[3]));
            dirTurnDesc = dirName + "出口" + turnDesc + laneNo + "车道";
            scopeList.add(dirTurnDesc);

            //补充缺少时段数据，保留时段字段默认值
            List<CrossLaneDataHistPoExtend> value = this.processData(entry, groupType, sortedSet, startTime);

            mapList.put("scopeCode", key);
            mapList.put("scopeName", dirTurnDesc);
            mapList.put("list", value);
            allList.add(mapList);

        }


        JSONObject jsonObject = new JSONObject();
        jsonObject.put("timeList", sortedSet);
        jsonObject.put("dataList", allList);
        jsonObject.put("scopeList", scopeList);

        return jsonObject;
    }

    @Override
    public JSONObject findCrossExitRoadDirInfo(String crossId, String startTime, String endTime, String groupType, Integer objectType) {
        Map<String, Object> params = new HashMap<>();
        params.put("crossId", crossId);
        params.put("startDate", startTime);
        params.put("endDate", endTime);
        params.put("groupType", groupType);

        //存放时段
        Set<String> sortedSet = EsDateIndexUtil.getTimeGranularityAxis(groupType, startTime, endTime);

        //======================================================================================================//
        //存放所有数据
        List<Map<String, Object>> allList = new ArrayList<>();

        List<CrossLaneDataHistPoExtend> list = new ArrayList<>();
        if (objectType == 1) {
            list = greenwaveHistoryMapper.findCrossExitRoadIndex(params);
        } else if (objectType == 2) {
            list = greenwaveHistoryMapper.findCrossExitRoadLaneIndex(params);
        }

        list = list.stream().sorted(Comparator.comparing(o -> o.getStartTime())).collect(Collectors.toList());
        //按时间分组分组
        Map<String, List<CrossLaneDataHistPoExtend>> groupByDir = list.stream().collect(Collectors.groupingBy(o -> o.getCrossId() + "_" + o.getLaneId() + "_" + o.getDirType() + "_" + o.getTurnType(), TreeMap::new, Collectors.toList()));

        Set<String> scopeList = new TreeSet<>();

        for (Map.Entry<String, List<CrossLaneDataHistPoExtend>> entry : groupByDir.entrySet()) {
            String key = entry.getKey();
            String[] sps = key.split("_");

            String laneId = sps[1];
            String laneNo = laneId.substring(laneId.length() - 2);
            Integer dirType = Integer.parseInt(sps[2]);
            String dirName = null;
            String turnDesc = null;
            String dirTurnDesc = null;

            Map<String, Object> mapList = new HashMap<>();

            mapList.put("laneId", laneId);
            mapList.put("laneNo", laneNo);
            if (objectType == 1) {
                dirName = BaseEnum.SignalDirectionEnum.getNameByCode(dirType);
                dirTurnDesc = dirName + "出口";
            } else if (objectType == 2) {
                dirName = BaseEnum.SignalDirectionEnum.getNameByCode(dirType);
                turnDesc = TurnConvertEnum.getDescByKey(Integer.valueOf(sps[3]));
                dirTurnDesc = dirName + "出口" + turnDesc + laneNo + "车道";
            }
            scopeList.add(dirTurnDesc);

            //补充缺少时段数据，保留时段字段默认值
            List<CrossLaneDataHistPoExtend> value = this.processData(entry, groupType, sortedSet, startTime);

            mapList.put("scopeCode", key);
            mapList.put("scopeName", dirTurnDesc);
            mapList.put("list", value);
            allList.add(mapList);
        }

        JSONObject jsonObject = new JSONObject();
        jsonObject.put("timeList", sortedSet);
        jsonObject.put("dataList", allList);
        jsonObject.put("scopeList", scopeList);

        return jsonObject;
    }

    /**
     * 补充缺少时段数据，保留时段字段默认值
     *
     * @param entry
     * @param groupType
     * @param sortedSet
     * @param startTime
     */
    private List<CrossLaneDataHistPoExtend> processData(Map.Entry<String, List<CrossLaneDataHistPoExtend>> entry, String groupType, Set<String> sortedSet, String startTime) {
        //按方向排序
        List<CrossLaneDataHistPoExtend> value = entry.getValue().stream().sorted(Comparator.comparing(o -> o.getStartTime())).collect(Collectors.toList());
        //========================路口级服务水平加权平均=================================================================//
        String serviceLevelList = value.get(0).getServiceLevel();
        StringBuilder serviceLevelBuf = new StringBuilder();
        if (serviceLevelList != null) {
           Map<String,Long> serviceGroup =  Arrays.stream(serviceLevelList.split(",")).collect(Collectors.groupingBy(String::valueOf,Collectors.counting()));
            double total = 0;
            for (Map.Entry<String, Long> entry1 : serviceGroup.entrySet()) {
                String key = entry1.getKey();
                total += ServiceLevelEnum.getByType(key).getVal() * entry1.getValue();
            }
            //服务水平加权平均计算：给服务水平(等级得分*该等级的数量)累加/总数量
            int avgService = (int) Math.round(total/value.size());
            String avgServiceLevel = ServiceLevelEnum.getByVal(avgService)==null?"A":ServiceLevelEnum.getByVal(avgService).getType();
            serviceLevelBuf.append(avgServiceLevel);
        }


        //有实际数据的时刻点
        Set<String> timeList = new HashSet<>();
        if (Objects.equals("4", groupType)) {
            //天粒度
            timeList.addAll(value.stream().map(po -> DateUtil.formatDate(po.getStartTime(), EsDateIndexUtil.YMD_FORMATTER)).collect(Collectors.toSet()));
        } else if (Objects.equals("3", groupType)) {
            //小时粒度
            timeList.addAll(value.stream().map(po -> DateUtil.formatDate(po.getStartTime(), EsDateIndexUtil.H_FORMATTER)).collect(Collectors.toSet()));
        } else {
            //小时分钟粒度
            timeList.addAll(value.stream().map(po -> DateUtil.formatDate(po.getStartTime(), EsDateIndexUtil.HM_FORMATTER)).collect(Collectors.toSet()));
        }
        //补充缺少时段数据，保留时段字段默认值
        for (String timeSec : sortedSet) {
            if (!timeList.contains(timeSec)) {
                CrossLaneDataHistPoExtend tmp = new CrossLaneDataHistPoExtend();
                tmp.setTimeAxis(timeSec);
                String parseTime = startTime.substring(0, startTime.lastIndexOf(" ") + 1) + timeSec;
                if (Objects.equals("4", groupType)) {
                    tmp.setStartTime(DateTime.parse(timeSec, DateTimeFormat.forPattern(EsDateIndexUtil.YMD_FORMATTER)).toDate());
                } else {
                    tmp.setStartTime(DateTime.parse(parseTime, DateTimeFormat.forPattern("yyyy-MM-dd HH:mm")).toDate());
                }
                value.add(tmp);
            }
        }
        value = value.stream().sorted(Comparator.comparing(CrossLaneDataHistPoExtend::getStartTime)).collect(Collectors.toList());



        //==========================================================================================================//
        //时间轴
        value.forEach(o -> {
            if (Objects.equals("4", groupType)) {
                o.setTimeAxis(DateUtil.formatDate(o.getStartTime(), EsDateIndexUtil.YMD_FORMATTER));
            } else if (Objects.equals("3", groupType)) {
                o.setTimeAxis(DateUtil.formatDate(o.getStartTime(), EsDateIndexUtil.H_FORMATTER));
            } else {
                o.setTimeAxis(DateUtil.formatDate(o.getStartTime(), EsDateIndexUtil.HM_FORMATTER));
            }
            o.setServiceLevel(serviceLevelBuf.toString());
        });

        return value;
    }


}
